VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ClsSerialPkt"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'Purpose
'   To communicate to devices on the serial port using a packet protocol
'   This class simultaneously arbitrates requests from separate code sections
'
'To send commands you need to...
'   1 use "SocketOpen" to create a socket to that DeviceID
'   2. Set requested status info bits. (This is OR'ed with other sockets)
'   3. Send commands or just listen for status
'   4. Call "SocketClose" When done.
'
Public Event Timer55() 'raised every timer tick
Public Event DataReceive(newData() As Byte, DataLen As Integer)
Public Event StatusReceived(DeviceID As Integer, StatusCode As Integer, newData() As Byte, DataLen As Integer)
Public Event DiagReceived(DeviceID As Integer, newData() As Byte, DataLen As Integer)
Public Event SocketUpdate(SocketID As Integer)
Public Event DeviceListUpdate() 'new device list

'----------------------------------------------------
Dim IamBusy%
Dim WithEvents MyComPort As FrmComPort
Attribute MyComPort.VB_VarHelpID = -1

Dim MyDeviceInfo(256) As DeviceInfoType 'one for each possible ID
Dim MySocketInfo(256) As SocketInfoType 'sockets arbritrate who is taking. Socekt(0) is reserved

Dim m_ActiveSocket As Integer       'Who sent the request
Dim m_ActiveNumber As Long          'Now serving x
Dim m_WaitingNumber As Long         'Who is in line

Dim m_ActiveDeviceID As Integer     'Where command was sent
Dim m_ActiveCommand As Integer      'what was the request
Dim m_ActiveBuffer(512) As Byte     'The command sent. Kept for resends
Dim m_ActiveBufferLen As Integer    '
Dim m_ActiveRespLen As Integer      'Expected response size in bytes

Dim m_ReceiveBuffer(512) As Byte  'Large enough to hold debug info
Dim m_ReceiveBufferLen As Integer 'current lenght in recieive buffer
Dim m_ReceiveState As Integer     'state of decoder
Dim m_ReceiveTimer As Integer     'for timeout

Private Sub Class_Initialize()
'
    FrmComPort.Show
    FrmComPort.Visible = False
        
    m_ActiveSocket = 0
    m_ActiveNumber = 0
    m_WaitingNumber = 0
    m_ActiveDeviceID = 0
    m_ActiveCommand = -1   'no command active
    m_ActiveBufferLen = 0
    m_ActiveRespLen = 0
    
    m_ReceiveBufferLen = 0
    m_ReceiveState = 0

    Set MyComPort = FrmComPort
    'MyComPort.BaudRate = 19200
End Sub

Private Sub Class_Terminate()
'
    Set MyComPort = Nothing
    
    Dim i%
    For i% = 0 To Forms.Count - 1
        If (Forms(i%) Is FrmComPort) Then
            Unload Forms(i%)
            Exit For
        End If
    Next i%
    
End Sub


Private Sub MyComPort_DataReceive(newData() As Byte, DataLen As Integer)
    Dim i%
    Dim Checksum%, ChkSumPkt%
    For i% = 0 To DataLen - 1
        m_ReceiveBuffer(m_ReceiveBufferLen) = newData(i)
        m_ReceiveBufferLen = m_ReceiveBufferLen + 1
        If m_ReceiveBufferLen > 512 Then m_ReceiveBufferLen = 0
    Next
    RaiseEvent DataReceive(newData(), DataLen) 'pass it on. If anyone cares?
    
    If m_ReceiveBufferLen >= m_ActiveRespLen Then
        If (m_ActiveRespLen >= 2) Then
            Checksum% = 0
            For i% = 0 To m_ActiveRespLen - 2
                Checksum% = Checksum% + m_ReceiveBuffer(i)
            Next
            Checksum% = Checksum% And &HFF
            ChkSumPkt% = m_ReceiveBuffer(m_ActiveRespLen - 1) And &HFF
            
            If (Checksum% = ChkSumPkt%) Then
            
                Select Case m_ActiveCommand
                Case -1: 'no command active
                Case &HD: 'debug info
                    'Hex dump data
                    RaiseEvent DiagReceived(m_ActiveDeviceID, m_ReceiveBuffer(), m_ReceiveBufferLen)
                Case 3: 'custom status
                    Call PStatusReceived(MyDeviceInfo(m_ActiveDeviceID).CustomStatusCode)
                Case Else 'Normal status
                    Call PStatusReceived(MyDeviceInfo(m_ActiveDeviceID).StatusCode)
                End Select
                
                MySocketInfo(m_ActiveSocket).Active = False
            End If
        End If
        
        m_ReceiveBufferLen = 0 'flush
    End If
    
    
End Sub
Private Sub PStatusReceived(ByVal StatusCode As Integer)
    Dim bOffset%
    bOffset% = 0
    
    MyDeviceInfo(m_ActiveDeviceID).StatusByte = GetValFromBuffer(bOffset%, 1)
    If StatusCode And 1 Then
        MyDeviceInfo(m_ActiveDeviceID).StatusPos = GetValFromBuffer(bOffset%, 4)
    End If
    If StatusCode And 2 Then
        MyDeviceInfo(m_ActiveDeviceID).StatusAD = GetValFromBuffer(bOffset%, 1)
    End If
    If StatusCode And 4 Then
        MyDeviceInfo(m_ActiveDeviceID).StatusVel = GetValFromBuffer(bOffset%, 2)
    End If
    If StatusCode And 8 Then
        MyDeviceInfo(m_ActiveDeviceID).StatusAux = GetValFromBuffer(bOffset%, 1)
    End If
    If StatusCode And &H10 Then
        MyDeviceInfo(m_ActiveDeviceID).StatusHome = GetValFromBuffer(bOffset%, 4)
    End If
    If StatusCode And &H20 Then
        MyDeviceInfo(m_ActiveDeviceID).StatusType = GetValFromBuffer(bOffset%, 2)
    End If
    
    RaiseEvent StatusReceived(m_ActiveDeviceID, StatusCode, m_ReceiveBuffer(), m_ReceiveBufferLen)
End Sub
Private Function GetValFromBuffer(bOffset%, ByVal Blen%) As Long
    Dim tmp&  'get a value from the buffer (LSO)
    'updates bOffset%
    Select Case Blen%
    Case 1: 'unsigned 8 bit integer
        tmp& = m_ReceiveBuffer(bOffset%)
    Case 2: 'signed 16 bit integer
        tmp& = m_ReceiveBuffer(bOffset%) + (m_ReceiveBuffer(bOffset% + 1) * &H100)
        If m_ReceiveBuffer(bOffset% + 1) And &H80 Then tmp& = tmp& Or &HFFFF8000
    Case 4: 'signed 32 bit integer
        tmp& = m_ReceiveBuffer(bOffset%)
        tmp& = tmp& + (m_ReceiveBuffer(bOffset% + 1) * &H100&)
        tmp& = tmp& + (m_ReceiveBuffer(bOffset% + 2) * &H10000)
        tmp& = tmp& + ((m_ReceiveBuffer(bOffset% + 3) And &H7F) * &H1000000)
        If m_ReceiveBuffer(bOffset% + 3) And &H80 Then tmp& = tmp& Or &H80000000
    End Select
    GetValFromBuffer = tmp&
    bOffset% = bOffset% + Blen%
End Function





Private Sub MyComPort_Timer55()
'
    If MySocketInfo(m_ActiveSocket).Active Then
        m_ReceiveTimer = m_ReceiveTimer + 1
        If m_ReceiveTimer > 2 Then
            m_ReceiveTimer = 0
            MySocketInfo(m_ActiveSocket).Active = False
            MySocketInfo(m_ActiveSocket).TimeOut = True
            m_ActiveCommand = -1
        End If
    End If
    RaiseEvent Timer55
End Sub

'-------------- Port interface --------------------------
Public Property Get PortOpen() As Boolean
    If Not MyComPort Is Nothing Then PortOpen = MyComPort.PortOpen
End Property
Public Property Let PortOpen(ByVal vNewValue As Boolean)
    If Not MyComPort Is Nothing Then MyComPort.PortOpen = vNewValue
End Property
Public Property Get CommPort() As Integer
    If Not MyComPort Is Nothing Then CommPort = MyComPort.CommPort
End Property
Public Property Let CommPort(ByVal vNewValue As Integer)
    If Not MyComPort Is Nothing Then MyComPort.CommPort = vNewValue
End Property

Public Property Get BaudRate() As Long
    If Not MyComPort Is Nothing Then BaudRate = MyComPort.BaudRate
End Property
Public Property Let BaudRate(ByVal vNewValue As Long)
    If Not MyComPort Is Nothing Then MyComPort.BaudRate = vNewValue
End Property

'------------ Low level Packet Commands -------------
'
Private Sub SendCommand(DeviceID As Integer, CmdCode As Integer, Dat() As Byte, CmdLen As Integer)
'Send a command
    Dim i%, ChkSum%
    m_ActiveDeviceID = DeviceID
    m_ActiveCommand = CmdCode
    m_ActiveBuffer(0) = &HAA
    m_ActiveBuffer(1) = DeviceID
    m_ActiveBuffer(2) = (CmdCode And &HF) Or ((CmdLen And &HF) * &H10)
    ChkSum% = m_ActiveBuffer(1)
    If CmdLen Then
        For i% = 0 To CmdLen - 1
            m_ActiveBuffer(i% + 3) = Dat(i%)
            ChkSum% = ChkSum% + m_ActiveBuffer(i% + 3)
        Next i%
    End If
    m_ActiveBuffer(CmdLen + 3) = (ChkSum% And &HFF)
    m_ActiveBufferLen = 3 + CmdLen + 1
    
    'set up receiver
    Select Case m_ActiveCommand
    Case -1: 'no command active
    Case &HD: 'debug info
        'm_ActiveRespLen
    Case 3: 'custom status
        m_ActiveRespLen = GetStatusLenght(MyDeviceInfo(DeviceID).CustomStatusCode)
    Case Else 'Normal status
        m_ActiveRespLen = GetStatusLenght(MyDeviceInfo(DeviceID).StatusCode)
    End Select
    
    m_ReceiveBufferLen = 0
    m_ReceiveState = 0
    m_ReceiveTimer = 0
    
    Call MyComPort.SendDataWLen(m_ActiveBuffer(), m_ActiveBufferLen)
End Sub
Private Sub SendFlush()
    Dim Dat(50) As Byte, i%
    For i = 0 To 20
        Dat(i) = 0
    Next
    Call MyComPort.SendDataWLen(Dat(), 20) 'flush
    'Call pause55(2)
    m_ReceiveBufferLen = 0
End Sub
'------------

'-------------------
Private Function GetStatusLenght(StatusCode As Integer)
'returns the number of bytes needed
    Dim Stlen%
    Stlen% = 1 'status byte
    If StatusCode And &H1 Then Stlen% = Stlen% + 4  'Current encoder Position, LSO
    If StatusCode And &H2 Then Stlen% = Stlen% + 1  'Send A/D value. (R/C input)
    If StatusCode And &H4 Then Stlen% = Stlen% + 2  'Current Velocity. (Counts per servo tick)
    If StatusCode And &H8 Then Stlen% = Stlen% + 1  'Auxiliary status byte
    If StatusCode And &H10 Then Stlen% = Stlen% + 4 'Saved encoder Position
    If StatusCode And &H20 Then Stlen% = Stlen% + 2 'Send Device ID (Atmel Servo = 0)
    GetStatusLenght = Stlen% + 1 'add checksum
End Function

'--------------------- port control ------------
Public Function PortInit(Port As Integer) As Boolean
    'opens the port and enumerates the ID numbers
    'If sucessful returns true when done
    Dim i%
    If IamBusy% Then Exit Function
    IamBusy% = True
    
    Dim ASocket As Integer
    ASocket = Me.SocketOpen(0)
    
    BaudRate = 115200                  'if the computer reset...
    CommPort = Port                    ' some devices may be still at 115200 so...
    PortOpen = True
    
    Call SendFlush
    Me.SocketDeviceID(ASocket) = 255
    Call Me.CmdHardReset(ASocket)      'send broadcast hard reset
    Call Me.SocketWaitComplete(ASocket)
        
    BaudRate = 19200                    'default baudrate
    Call SendFlush
    Me.SocketDeviceID(ASocket) = 255
    Call Me.CmdHardReset(ASocket)       'reset any new devices or current devices
    Call Me.SocketWaitComplete(ASocket)
    Call SendFlush
        
    For i = 0 To 127
        MyDeviceInfo(i).Found = False
        MyDeviceInfo(i).StatusCode = 0
        m_ActiveBuffer(i) = 0
    Next
   
    Me.SocketDeviceID(ASocket) = 0
    For i = 1 To 127
        Call CmdSetAddress(ASocket, i, 255)
        Call SocketWaitComplete(ASocket)
        
        If SocketTimeout(ASocket) Then
            Call CmdSetAddress(ASocket, i, 255)
            Call SocketWaitComplete(ASocket)
        
           If SocketTimeout(ASocket) Then Exit For
        End If
        'Call SendFlush
        Call pause55(2)
        MyDeviceInfo(i).Found = True
    Next
    
    Me.SocketDeviceID(ASocket) = 255    'all devices
    Call Me.CmdSetBaudRate(ASocket, 10) '115200
    Call SocketWaitComplete(ASocket)
    BaudRate = 115200                  '
    Call SendFlush
    
    'poll all status info
            
    Call SocketClose(ASocket)
    RaiseEvent DeviceListUpdate
    
    IamBusy% = False
    
    PortInit = True
End Function
'-------------------- Packet Interface ---------------
Public Function SocketOpen(DeviceID As Integer)
'returns a socket handle
'or -1 if none left
    Dim i%
    SocketOpen = -1
    For i% = 1 To 256
        If MySocketInfo(i).SOpen = False Then
            MySocketInfo(i).SOpen = True
            MySocketInfo(i).DeviceID = DeviceID
            SocketOpen = i
            Exit For
        End If
    Next i%
End Function
Public Sub SocketClose(SocketID As Integer)
    If (SocketID >= 1) And (SocketID <= 256) Then
        MySocketInfo(SocketID).SOpen = False
    End If
End Sub
Public Property Get SocketDeviceID(SocketID As Integer) As Integer
    If (SocketID >= 1) And (SocketID <= 256) Then
        SocketDeviceID = MySocketInfo(SocketID).DeviceID
    End If
End Property
Public Property Let SocketDeviceID(SocketID As Integer, VnewVal As Integer)
    If (SocketID >= 1) And (SocketID <= 256) Then
        MySocketInfo(SocketID).DeviceID = VnewVal
    End If
End Property


'-------------- Commands --------------------------------
Public Sub CmdSetAddress(SocketID As Integer, NewID As Integer, GroupID As Integer)
'0x01
    'take a number
    
    'ok now send
    m_ActiveSocket = SocketID
    MySocketInfo(SocketID).Active = True
    MySocketInfo(SocketID).TimeOut = False
    'm_ActiveNumber
    'm_WaitingNumber
    
    Dim Dat(10) As Byte
    Dat(0) = NewID
    Dat(1) = GroupID
    Call SendCommand(MySocketInfo(SocketID).DeviceID, &H1, Dat(), 2)

End Sub
Public Sub CmdGetCustomStatus(SocketID%, StatusCode%) '0x03
    
    'take a number
    
    'ok now send
    m_ActiveSocket = SocketID
    MySocketInfo(SocketID).Active = True
    MySocketInfo(SocketID).TimeOut = False
    'm_ActiveNumber
    'm_WaitingNumber
    
    Dim Dat(10) As Byte
    Dat(0) = StatusCode%
    MyDeviceInfo(MySocketInfo(SocketID).DeviceID).CustomStatusCode = StatusCode%
    Call SendCommand(MySocketInfo(SocketID).DeviceID, &H3, Dat(), 1)

End Sub

Public Sub CmdSetBaudRate(SocketID As Integer, BaudCode As Integer) '0x0A
' BaudCode = '9600=129, 19200=63, 38400=30, 57600=20, 115200=10
    'take a number
    
    'ok now send
    m_ActiveSocket = SocketID
    MySocketInfo(SocketID).Active = True
    MySocketInfo(SocketID).TimeOut = False
    'm_ActiveNumber
    'm_WaitingNumber
    
    Dim Dat(10) As Byte
    Dat(0) = BaudCode
    Call SendCommand(MySocketInfo(SocketID).DeviceID, &HA, Dat(), 1)

    

End Sub
Public Sub CmdGetStatus(SocketID As Integer) '0x0E
    'take a number
    
    'ok now send
    m_ActiveSocket = SocketID
    MySocketInfo(SocketID).Active = True
    MySocketInfo(SocketID).TimeOut = False
    'm_ActiveNumber
    'm_WaitingNumber
    
    Dim Dat(10) As Byte
    Call SendCommand(MySocketInfo(SocketID).DeviceID, &HE, Dat(), 0)

End Sub


Public Sub CmdHardReset(SocketID As Integer) '0x0F
    'take a number
    
    'ok now send
    m_ActiveSocket = SocketID
    MySocketInfo(SocketID).Active = True
    MySocketInfo(SocketID).TimeOut = False
    'm_ActiveNumber
    'm_WaitingNumber
    
    Dim Dat(10) As Byte
    Call SendCommand(MySocketInfo(SocketID).DeviceID, &HF, Dat(), 0)

End Sub

Public Sub SocketWaitComplete(SocketID As Integer)
'wait until socket is done
    If (SocketID >= 1) And (SocketID <= 256) Then
        Do
            DoEvents
            If FrmComPort Is Nothing Then Exit Do
        Loop While (MySocketInfo(SocketID).Active = True)
    End If
End Sub

Public Property Get SocketTimeout(SocketID As Integer) As Boolean
    If (SocketID >= 1) And (SocketID <= 256) Then
        SocketTimeout = MySocketInfo(SocketID).TimeOut
    End If
End Property
Public Property Get SocketActive(SocketID As Integer) As Boolean
    If (SocketID >= 1) And (SocketID <= 256) Then
        SocketActive = MySocketInfo(SocketID).Active
    End If
End Property

'------------------ Diagnostic interface ----------------
'this should work!!!
'Public Property Get DeviceInfo(ByVal DeviceID As Integer) As DeviceInfoType
    'If DeviceID < 0 Or DeviceID > 255 Then DeviceID = 0
    'DeviceInfo = MyDeviceInfo(DeviceID)
'End Property
'Public Property Get SocketInfo(ByVal SocketNum As Integer) As SocketInfoType
    'If SocketNum < 1 Or SocketNum > 255 Then SocketNum = 1
    'SocketInfo= MySocketInfo(SocketNum)
'End Property
'stupid basic bug :(
Public Function GetDeviceInfoFound(ByVal DeviceID As Integer) As Boolean
    If DeviceID < 0 Or DeviceID > 255 Then DeviceID = 0
    GetDeviceInfoFound = MyDeviceInfo(DeviceID).Found
End Function
Public Function GetDeviceInfoByNum(ByVal DeviceID As Integer, StatNum%) As Long
    If DeviceID < 0 Or DeviceID > 255 Then DeviceID = 0
    Select Case StatNum%
    Case 0: GetDeviceInfoByNum = MyDeviceInfo(DeviceID).StatusByte
    Case 1: GetDeviceInfoByNum = MyDeviceInfo(DeviceID).StatusPos
    Case 2: GetDeviceInfoByNum = MyDeviceInfo(DeviceID).StatusAD
    Case 4: GetDeviceInfoByNum = MyDeviceInfo(DeviceID).StatusVel
    Case 8: GetDeviceInfoByNum = MyDeviceInfo(DeviceID).StatusAux
    Case &H10: GetDeviceInfoByNum = MyDeviceInfo(DeviceID).StatusHome
    Case &H20: GetDeviceInfoByNum = MyDeviceInfo(DeviceID).StatusType
    End Select
End Function


Private Sub pause55(tCnts%)
    Dim Tnew#, Told#, Cnter%
    
    For Cnter% = 0 To tCnts%
        Tnew# = Timer
        Told# = Tnew#
        Do
            Tnew# = Timer
            DoEvents
        Loop While (Abs(Tnew# - Told#) < 0.01)
    Next
    
End Sub



